/*
 *  ZThreads, a platform-independant, multithreading and 
 *  synchroniation library
 *
 *  Copyright (C) 2001, 2002 Eric Crahen, See LGPL.TXT for details
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  SUNY @ Buffalo, hereby disclaims all copyright interest in the
 *  ZThreads library written by Eric Crahen
 */

#include "zthread/AbstractThreadLocal.h"
#include "ValueHolder.h"

#ifdef _WIN32
#include "Win32_ThreadLocalImpl.h"
#elif _POSIX
#include "POSIX_ThreadLocalImpl.h"
#endif


using namespace ZThread;

AbstractThreadLocal::AbstractThreadLocal() throw() 
  : _impl(new ThreadLocalImpl()) {
  
}


AbstractThreadLocal::~AbstractThreadLocal() throw() {
  delete _impl;
}

void* AbstractThreadLocal::get() const 
  throw(Synchronization_Exception) {  

  // Get the value holder for the current thread
  ValueHolder* holder = _impl->getValueHolder(this);
  return holder->getValue();

}


void* AbstractThreadLocal::set(void* newValue) const 
  throw(Synchronization_Exception) {

  // Get the value holder for the current thread
  ValueHolder* holder = _impl->getValueHolder(this);
  return holder->setValue(newValue);

}




void* AbstractThreadLocal::fetch() const 
  throw(Synchronization_Exception) {  

  return _impl->get();

}

void* AbstractThreadLocal::put(void* value) const 
  throw(Synchronization_Exception) {  

  return _impl->set(value);

}


