/*
 *  ZThreads, a platform-independant, multithreading and 
 *  synchroniation library
 *
 *  Copyright (C) 2001, 2002 Eric Crahen, See LGPL.TXT for details
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  SUNY @ Buffalo, hereby disclaims all copyright interest in the
 *  ZThreads library written by Eric Crahen
 */

#ifndef __ZTCONDITIONIMPL_H__
#define __ZTCONDITIONIMPL_H__

#include "zthread/FastMutex.h"
#include "zthread/SynchronizationExceptions.h"
#include "zthread/Lockable.h"
#include "Synchronized.h"

#include <vector>

namespace ZThread {
  
  class ThreadImpl;

/**
 * @class ConditionImpl
 * @author Eric Crahen <crahen@cse.buffalo.edu>
 * @date <2002-03-06T07:36:39-0500>
 * @version 2.0.4
 *
 * This synchronization object provides a classic condition variable.
 */ 
class ZTHREAD_API ConditionImpl : public Synchronized {
    
  typedef std::vector<MONITOR*> WaiterList;
  
  //! Waiters currently blocked
  WaiterList _waiters;

  //! Serialize access to this object
  FastMutex _lock;
  
  public:
  
  ConditionImpl() 
    throw(Synchronization_Exception);

  ~ConditionImpl() throw();
  
  void signal() 
    throw(Synchronization_Exception);

  void broadcast() 
    throw(Synchronization_Exception);
  
  void wait(Lockable&) 
    throw(Synchronization_Exception);
  
  bool wait(Lockable&, unsigned long) 
    throw(Synchronization_Exception);
  
}; /* ConditionImpl */



}; 

#endif
