/*
 *  ZThreads, a platform-independant, multithreading and 
 *  synchroniation library
 *
 *  Copyright (C) 2001, 2002 Eric Crahen, See LGPL.TXT for details
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  SUNY @ Buffalo, hereby disclaims all copyright interest in the
 *  ZThreads library written by Eric Crahen
 */

#ifndef __ZTMUTEXIMPL_H__
#define __ZTMUTEXIMPL_H__

#include "zthread/FastMutex.h"
#include "zthread/SynchronizationExceptions.h"
#include "Synchronized.h"

#include <vector>

namespace ZThread {

/**
 * @class MutexImpl
 * @author Eric Crahen <crahen@cse.buffalo.edu>
 * @date <2002-03-06T07:36:01-0500>
 * @version 2.0.4
 *
 * This synchronization object provides serialized access 
 * through an acquire/release protocol. 
 */
class ZTHREAD_API MutexImpl : public Synchronized {
  
  typedef std::vector<MONITOR*> WaiterList;  

  //! List of Events that are waiting for notification 
  WaiterList _waiters;

  //! Serialize access to this Mutex
  FastMutex _lock;

  //! Current owning Event object
  MONITOR* _owner;
  
  //! Increment 
  size_t _increment;

  //! Entry count 
  size_t _count;

  public:
   
  MutexImpl(size_t increment=0) 
    throw(Synchronization_Exception);

  virtual ~MutexImpl()
    throw();
  
  void acquire() 
    throw(Synchronization_Exception);
  
  bool tryAcquire(unsigned long) 
    throw(Synchronization_Exception);
  
  void release() 
    throw(Synchronization_Exception);

}; /* MutexImpl */


}; 

#endif
