/*
 *  ZThreads, a platform-independant, multithreading and 
 *  synchroniation library
 *
 *  Copyright (C) 2001, 2002 Eric Crahen, See LGPL.TXT for details
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  SUNY @ Buffalo, hereby disclaims all copyright interest in the
 *  ZThreads library written by Eric Crahen
 */


#ifndef __ZTTHREADLOCALIMPL_H__
#define __ZTTHREADLOCALIMPL_H__

#include "zthread/ZThread.h"
#include "zthread/AbstractThreadLocal.h"

#include <pthread.h>

namespace ZThread {

  class ValueHolder;

/**
 * @class ThreadLocalImpl
 * @author Eric Crahen <crahen@cse.buffalo.edu>
 * @date <2002-02-17T20:05:55-0500>
 * @version 2.0.0a
 */
class ThreadLocalImpl {

  pthread_key_t _key;
  bool _valid;

  public:

  ThreadLocalImpl() throw();  
  ~ThreadLocalImpl() throw();
  
  /**
   * Get the TLS value for the current thread. If this value has not been 
   * accessed from the current thread, then invoke the initialize() function 
   * on the AbstractThreadLocal passed in.
   *
   * @param AbstractThreadLocal* - object to keep track of 
   */  
  ValueHolder* getValueHolder(const AbstractThreadLocal*) const
    throw(Synchronization_Exception);

  /**
   *
   */
  void* get() const
    throw(Synchronization_Exception);

  /**
   *
   */
  void* set(void*) const
    throw(Synchronization_Exception);


#ifndef NDEBUG

  static void cleanup();

 private:

  static bool cleanedUp;

#endif

};


};


#endif
