/*
 *  ZThreads, a platform-independant, multithreading and 
 *  synchroniation library
 *
 *  Copyright (C) 2001, 2002 Eric Crahen, See LGPL.TXT for details
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  SUNY @ Buffalo, hereby disclaims all copyright interest in the
 *  ZThreads library written by Eric Crahen
 */


#ifndef __ZTPOSIXTHREADMANAGER_H__
#define __ZTPOSIXTHREADMANAGER_H__

#include "zthread/POSIX_FastMutex.h"
#include "zthread/Singleton.h"

#include <map>
#include <utility>

namespace ZThread {

class ThreadImpl;
  
/**
 * @class ThreadManager
 * @version 2.0.1
 * @author Eric Crahen <crahen@cse.buffalo.edu>
 * @date <2002-03-03T06:01:21-0500>
 *
 * An instance of ThreadManger starts when the first Thread is used. This
 * instance is responsible for DAEMON and REFERENCE threads. Signals used  
 * for I/O interruption and cleanup handlers are also dealt with by the 
 * ThreadManager.
 */
class ThreadManager : public Singleton<ThreadManager, FastMutex> {
  
 protected:

  typedef std::map<pthread_t, ThreadImpl*> ThreadMap;
  
 public:

  ThreadManager() throw();
  ~ThreadManager() throw();
  
  void referenceThread(ThreadImpl*);
  
  void attachThread(ThreadImpl*);
  void detachThread(ThreadImpl*);

 private:

  static void fatal(const char*);

  static void cleanup();

  static void interruptHandler(int);

  //! Map of detached (DAEMON) threads
  ThreadMap _detached;

  //! Map of discovered (REFERENCE) threads. 
  ThreadMap _reference;

  //! Serialize access to the maps.
  FastMutex _mapLock;
  
}; 

};

#endif
