/*
 *  ZThreads, a platform-independant, multithreading and 
 *  synchroniation library
 *
 *  Copyright (C) 2001, 2002 Eric Crahen, See LGPL.TXT for details
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  SUNY @ Buffalo, hereby disclaims all copyright interest in the
 *  ZThreads library written by Eric Crahen
 */

#ifndef __ZTSEMAPHOREIMPL_H__
#define __ZTSEMAPHOREIMPL_H__

#include "zthread/FastMutex.h"
#include "zthread/SynchronizationExceptions.h"
#include "Synchronized.h"

#include <vector>

namespace ZThread {

class ThreadImpl;

/**
 * @class SemaphoreImpl
 * @author Eric Crahen <crahen@cse.buffalo.edu>
 * @date <2002-03-06T07:36:28-0500>
 * @version 2.0.4
 *
 * This class implements a classical semaphore using an acquire/release protocol.
 */
class ZTHREAD_API SemaphoreImpl : public Synchronized {

  typedef std::vector<MONITOR*> WaiterList;  

  //! List of waiting events
  WaiterList _waiters;

  //! Serialize access to this object
  FastMutex _lock;

  //! Current count
  int _count;

  //! Maximum count if any
  int _maxCount;

  //! Flag for bounded or unbounded count
  bool _checked;

  public:
   
  SemaphoreImpl(int, unsigned int, bool checked = true) 
    throw(Synchronization_Exception);

  ~SemaphoreImpl()
    throw();
  
  void acquire() 
    throw(Synchronization_Exception);
  
  bool tryAcquire(unsigned long) 
    throw(Synchronization_Exception);
  
  void release() 
    throw(Synchronization_Exception);
  
  int count()
    throw();

}; /* SemaphoreImpl */


}; 

#endif
